#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-12
# Language: Python3

################################### IMPORTS ####################################

# Standard library
# Your imports from the standard library go here


# External imports
# Your imports from other packages go here


# Internal imports
from compass.ast.await_statement import AwaitStatement
from compass.ast.each import Each
from compass.ast.emit_statement import EmitStatement
from compass.ast.expression import Expression
from compass.ast.extern import Extern
from compass.ast.if_statement import IfStatement
from compass.ast.input_declaration import InputDeclaration
from compass.ast.local_statement import LocalStatement
from compass.ast.module import Module
from compass.ast.node import Node
from compass.ast.number import Number
from compass.ast.output_declaration import OutputDeclaration
from compass.ast.par import Par
from compass.ast.seq import Seq
from compass.ast.signal import Signal
from compass.ast.signal_expression import SignalExpression
from compass.ast.declaration import Declaration
from compass.ast.statement import Statement
from compass.ast.submodule import Submodule
from compass.ast.tree import Tree
from compass.ast.unary import *
from compass.ast.binary import *

################################### CLASSES ####################################

# Your classes go here

################################## FUNCTIONS ###################################

# Your functions go here

##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################
