#!/usr/bin/env python3

################################### METADATA ###################################

# Contributors: roadelou
# Contacts:
# Creation Date: 2021-03-12
# Language: Python3

################################### IMPORTS ####################################

# Standard library
# Your imports from the standard library go here


# External imports
# Your imports from other packages go here


# Internal imports
from compass.frontend.driver import compass_compile
from compass.frontend.command_line import command_line
# We give a more explicit name to the compass-builder entry point.
from compass.frontend.builder import command_line as compass_builder

################################### CLASSES ####################################

# Your classes go here

################################## FUNCTIONS ###################################

# Your functions go here

##################################### MAIN #####################################

if __name__ == "__main__":
    # The code to run when this file is used as a script goes here
    pass

##################################### EOF ######################################
